function [FF, BB, UU, Sigma_u] = genFPM_nondiag(N,T)
    % factor pricing model
    % non-Diagonal case
    
    % 1) generating factors
    %-------------------
    
    Mu_f=[0.026;0.0211;-0.0043];
    Sigma_f=[3.2351,0.1783,0.7783;0.1783,0.5069,0.0102;0.7783,0.0102,0.6586];
    R_f=chol(Sigma_f);

    FF = repmat(Mu_f,1,T) + R_f'*randn(3,T);
    
    % 2) factor loadings
    %---------------------
    Mu_b=[0.9833,-0.1233,0.0839]';
    Sigma_b=[0.0921,-0.0178,0.0436;-0.0178,0.0862,-0.0211;0.0436,-0.0211,0.7624];
    R_b=chol(Sigma_b);

    BB = repmat(Mu_b,1,N) + R_b'*randn(3,N);
    
    
    % 3) generating covariance matrix
    %--------------------------------
    
    % block diagonal matrix
    Sigma_u = zeros(N,N);
    N1 = 5;
    for iBlock=1:N/N1   % N1 is size of block
        a = randn(N1,1)*0.1;
        b = randn(N1,1)*0.1;
        c = randn(N1,1)*0.1;
        % set up block 
        SubBlock = zeros(N1,N1);
        for i=3:N1-1
            SubBlock(i+1,i+1) = 1+a(i)^2+b(i-1)^2+c(i-2)^2;
            SubBlock(i+1,i) = a(i)+b(i-1)*a(i-1)+c(i-2)*b(i-2);
            SubBlock(i,i+1) = SubBlock(i+1,i);
            SubBlock(i+1,i-1) = b(i-1)+c(i-2)*a(i-2);
            SubBlock(i-1,i+1) = SubBlock(i+1,i-1);
            SubBlock(i+1, i-2) = c(i-2);
            SubBlock(i-2,i+1) = SubBlock(i+1,i-2);
        end
        SubBlock(1,1) = 1; SubBlock(2,2) = 1+a(1)^2; SubBlock(3,3) = 1+a(2)^2+b(1)^2;
        SubBlock(1,2) = a(1); SubBlock(2,1) = SubBlock(1,2);
        SubBlock(1,3) = b(1); SubBlock(3,1) = SubBlock(1,3);
        SubBlock(2,3) = a(2)+a(1)*b(1); SubBlock(3,2) = SubBlock(2,3);

        tmp_range = (iBlock-1)*N1+1:(iBlock-1)*N1+5;
        Sigma_u(tmp_range,tmp_range) = SubBlock;
        clear SubBlock a b c;
    end
      
    
    R_u = chol(Sigma_u);
    UU = R_u'*randn(N,T);


end