function [Sigma,rho2]=sparseMatrixEstimator(u,C)
% return thresholded matrix
% u: N by T error panel
% C: threshold constant, threshold correlation matrix
% Sigma -- estimated covariance
% rho2 -- average squared correlation, see Pesaran and Yamagata (2012)
[N,T] = size(u);
rate = 1/sqrt(N)+sqrt(log(N))/sqrt(T);
Su = u*u'/(T-4);

if N > T*0.3
    % Covariance Matrix
    SuDiag = diag(diag(Su));
    SuDiagHalf = SuDiag^(1/2);
    R = SuDiagHalf\Su/SuDiagHalf;
    %R = corrcov(Su);
    th = abs(C)*rate;
    
    Rthresh = zeros(N,N);
    Ridx1 = abs(R) >= th;
    Rthresh(Ridx1) = sign(R(Ridx1)).*(abs(R(Ridx1))-th);
    Rthresh(logical(eye(N))) = R(logical(eye(N)));
    Sigma = SuDiagHalf*Rthresh*SuDiagHalf;
    
    % Correlation
    v = T-4;
    dd = 1-0.05/(N-1);
    ct = norminv(dd)^2;  % theta in PY
    %Rnew=eye(N);
    Rnew = R.*(v*R.^2 > ct);
    Rnew(logical(eye(N))) = ones(N,1);
    rho2 = (norm(Rnew,'fro')^2-N)/(N*(N-1));  
    
else
    Sigma = Su;
    rho2 = 0;
end;